﻿// Fill out your copyright notice in the Description page of Project Settings.


#include "EnemyCharacter.h"

#include "AIControlHub.h"
#include "AIController.h"
#include "BehaviorTree/BlackboardComponent.h"
#include "EndlessVendetta/EndlessVendettaCharacter.h"


class UAISense_Sight;
// Sets default values
AEnemyCharacter::AEnemyCharacter()
{
	// Set this character to call Tick() every frame.  You can turn this off to improve performance if you don't need it.
	PrimaryActorTick.bCanEverTick = true;
}

// Called when the game starts or when spawned
void AEnemyCharacter::BeginPlay()
{
	Super::BeginPlay();
	CharacterName = "Enemy";

	if (AEndlessVendettaCharacter* PlayerCharacter = Cast<AEndlessVendettaCharacter>(GetWorld()->GetFirstPlayerController()->GetPawn()))
	{
		PlayerCharacter->RestrictedAreaStatusChanged.AddUniqueDynamic(this, &AEnemyCharacter::SetIfPlayerIsInRestrictedArea);
	}
}

void AEnemyCharacter::OnDeath()
{
	Super::OnDeath();
	AlertLevelDelegateHandle.Reset();
	HuntPlayerDelegateHandle.Reset();
}

// Called every frame
void AEnemyCharacter::Tick(float DeltaTime)
{
	Super::Tick(DeltaTime);
}

void AEnemyCharacter::SubscribeToGroupAIEvents(AAIControlHub* ControlHub)
{
	DelegatedControlHub = ControlHub;
	AlertLevelDelegateHandle = ControlHub->AlertLevelEvent.AddUObject(this, &AEnemyCharacter::SetAlertLevel);
	HuntPlayerDelegateHandle = ControlHub->HuntPlayerEvent.AddUObject(this, &AEnemyCharacter::HuntPlayer);
}

void AEnemyCharacter::SetAlertLevel(const int NewAlertLevel) const
{
	if (!IsValid(this)) return;
	if (!IsValid(GetController())) return;
	Cast<AAIController>(GetController())->GetBlackboardComponent()->SetValueAsInt("AlertLevel", NewAlertLevel);
}

void AEnemyCharacter::SetLocalAlertLevel(int NewAlertLevel)
{
	if (!IsValid(GetController())) return;
	Cast<AAIController>(GetController())->GetBlackboardComponent()->SetValueAsBool("IsInvestigating", true);
	if (NewAlertLevel >= 2) SetHostilityLevel(EHostilityLevel::Hostile);
	if (!IsValid(DelegatedControlHub)) return;
	DelegatedControlHub->SetAlertLevel(NewAlertLevel);
}

void AEnemyCharacter::SetHostilityLevel(const EHostilityLevel NewHostilityLevel)
{
	if (NewHostilityLevel == EHostilityLevel::Hostile)
	{
		if (!IsValid(GetController())) return;
		Cast<AAIController>(GetController())->GetBlackboardComponent()->SetValueAsBool("IsHostile", true);
	}
	HostilityLevel = NewHostilityLevel;
}

float AEnemyCharacter::TakeDamage(float DamageAmount, FDamageEvent const& DamageEvent, AController* EventInstigator, AActor* DamageCauser)
{
	if (Cast<AEndlessVendettaCharacter>(EventInstigator->GetPawn()))
	{
		SetHostilityLevel(EHostilityLevel::Hostile);
		HuntPlayer(Cast<AEndlessVendettaCharacter>(EventInstigator->GetPawn())->GetActorLocation());
	}
	return Super::TakeDamage(DamageAmount, DamageEvent, EventInstigator, DamageCauser);
}

void AEnemyCharacter::EquipWeapon_Implementation()
{
}

void AEnemyCharacter::DeEquipWeapon_Implementation()
{
}

void AEnemyCharacter::SetFiring_Implementation(bool IsFiring)
{
}

void AEnemyCharacter::HuntPlayer(FVector PlayerLastKnownLocation)
{
	if (!IsValid(this)) return;
	if (!IsValid(GetController())) return;
	SetAlertLevel(2);
	EquipWeapon();
	Cast<AAIController>(GetController())->GetBlackboardComponent()->SetValueAsVector("LastKnownLocation", PlayerLastKnownLocation);
}

void AEnemyCharacter::SetIfPlayerIsInRestrictedArea(const bool bIsInRestrictedArea)
{
	if (!IsValid(GetController())) return;
	Cast<AAIController>(GetController())->GetBlackboardComponent()->SetValueAsBool("PlayerIsInRestrictedBounds", bIsInRestrictedArea);
}
